@@ -23,11 +23,13 @@ import org.apache.hudi.callback.client.http.HoodieWriteCommitHttpCallbackClient;
 import org.apache.http.StatusLine;
 import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.impl.client.CloseableHttpClient;
-import org.apache.log4j.AppenderSkeleton;
-import org.apache.log4j.Level;
-import org.apache.log4j.Logger;
-import org.apache.log4j.spi.LoggingEvent;
+import org.apache.logging.log4j.Level;
+import org.apache.logging.log4j.LogManager;
+import org.apache.logging.log4j.core.Appender;
+import org.apache.logging.log4j.core.LogEvent;
+import org.apache.logging.log4j.core.Logger;
 import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
 import org.mockito.ArgumentCaptor;
@@ -52,10 +54,10 @@ import static org.mockito.Mockito.when;
 public class TestCallbackHttpClient {
 
   @Mock
-  AppenderSkeleton appender;
+  Appender appender;
 
   @Captor
-  ArgumentCaptor<LoggingEvent> logCaptor;
+  ArgumentCaptor<LogEvent> logCaptor;
 
   @Mock
   CloseableHttpClient httpClient;
@@ -66,8 +68,17 @@ public class TestCallbackHttpClient {
   @Mock
   StatusLine statusLine;
 
+  @BeforeEach
+  void prepareAppender() {
+    when(appender.getName()).thenReturn("MockAppender");
+    when(appender.isStarted()).thenReturn(true);
+    when(appender.isStopped()).thenReturn(false);
+    ((Logger) LogManager.getLogger(HoodieWriteCommitHttpCallbackClient.class)).addAppender(appender);
+  }
+
   @AfterEach
   void resetMocks() {
+    ((Logger) LogManager.getLogger(HoodieWriteCommitHttpCallbackClient.class)).removeAppender(appender);
     reset(appender, httpClient, httpResponse, statusLine);
   }
 
@@ -83,21 +94,19 @@ public class TestCallbackHttpClient {
 
   @Test
   public void sendPayloadShouldLogWhenRequestFailed() throws IOException {
-    Logger.getRootLogger().addAppender(appender);
     when(httpClient.execute(any())).thenThrow(IOException.class);
 
     HoodieWriteCommitHttpCallbackClient hoodieWriteCommitCallBackHttpClient =
         new HoodieWriteCommitHttpCallbackClient("fake_api_key", "fake_url", httpClient);
     hoodieWriteCommitCallBackHttpClient.send("{}");
 
-    verify(appender).doAppend(logCaptor.capture());
-    assertEquals("Failed to send callback.", logCaptor.getValue().getRenderedMessage());
+    verify(appender).append(logCaptor.capture());
+    assertEquals("Failed to send callback.", logCaptor.getValue().getMessage().getFormattedMessage());
     assertEquals(Level.WARN, logCaptor.getValue().getLevel());
   }
 
   @Test
   public void sendPayloadShouldLogUnsuccessfulSending() {
-    Logger.getRootLogger().addAppender(appender);
     mockResponse(401);
     when(httpResponse.toString()).thenReturn("unauthorized");
 
@@ -105,22 +114,21 @@ public class TestCallbackHttpClient {
         new HoodieWriteCommitHttpCallbackClient("fake_api_key", "fake_url", httpClient);
     hoodieWriteCommitCallBackHttpClient.send("{}");
 
-    verify(appender).doAppend(logCaptor.capture());
-    assertEquals("Failed to send callback message. Response was unauthorized", logCaptor.getValue().getRenderedMessage());
+    verify(appender).append(logCaptor.capture());
+    assertEquals("Failed to send callback message. Response was unauthorized", logCaptor.getValue().getMessage().getFormattedMessage());
     assertEquals(Level.WARN, logCaptor.getValue().getLevel());
   }
 
   @Test
   public void sendPayloadShouldLogSuccessfulSending() {
-    Logger.getRootLogger().addAppender(appender);
     mockResponse(202);
 
     HoodieWriteCommitHttpCallbackClient hoodieWriteCommitCallBackHttpClient =
         new HoodieWriteCommitHttpCallbackClient("fake_api_key", "fake_url", httpClient);
     hoodieWriteCommitCallBackHttpClient.send("{}");
 
-    verify(appender).doAppend(logCaptor.capture());
-    assertTrue(logCaptor.getValue().getRenderedMessage().startsWith("Sent Callback data"));
+    verify(appender).append(logCaptor.capture());
+    assertTrue(logCaptor.getValue().getMessage().getFormattedMessage().startsWith("Sent Callback data"));
     assertEquals(Level.INFO, logCaptor.getValue().getLevel());
   }
 
