@@ -21,10 +21,10 @@ package org.apache.hudi.metrics;
 
 import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.exception.HoodieException;
+import org.apache.hudi.metrics.custom.CustomizableMetricsReporter;
 
 import com.codahale.metrics.MetricRegistry;
-import org.apache.hudi.exception.HoodieException;
-import org.apache.hudi.metrics.userdefined.AbstractUserDefinedMetricsReporter;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
 import org.mockito.Mock;
@@ -63,7 +63,7 @@ public class TestMetricsReporterFactory {
 
     when(config.getProps()).thenReturn(props);
     MetricsReporter reporter = MetricsReporterFactory.createReporter(config, registry);
-    assertTrue(reporter instanceof AbstractUserDefinedMetricsReporter);
+    assertTrue(reporter instanceof CustomizableMetricsReporter);
     assertEquals(props, ((DummyMetricsReporter) reporter).getProps());
     assertEquals(registry, ((DummyMetricsReporter) reporter).getRegistry());
   }
@@ -75,7 +75,7 @@ public class TestMetricsReporterFactory {
     assertThrows(HoodieException.class, () -> MetricsReporterFactory.createReporter(config, registry));
   }
 
-  public static class DummyMetricsReporter extends AbstractUserDefinedMetricsReporter {
+  public static class DummyMetricsReporter extends CustomizableMetricsReporter {
 
     public DummyMetricsReporter(Properties props, MetricRegistry registry) {
       super(props, registry);
