@@ -39,6 +39,7 @@ import org.apache.hudi.common.table.timeline.dto.FileSliceDTO;
 import org.apache.hudi.common.table.timeline.dto.InstantDTO;
 import org.apache.hudi.common.table.timeline.dto.TimelineDTO;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.RetryHelper;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.common.util.collection.Pair;
@@ -132,22 +133,35 @@ public class RemoteHoodieTableFileSystemView implements SyncableFileSystemView,
 
   private boolean closed = false;
 
+  private RetryHelper<Response> retryHelper;
+
+  private final HttpRequestCheckedFunction urlCheckedFunc;
+
   private enum RequestMethod {
     GET, POST
   }
 
   public RemoteHoodieTableFileSystemView(String server, int port, HoodieTableMetaClient metaClient) {
-    this(server, port, metaClient, 300);
+    this(metaClient, FileSystemViewStorageConfig.newBuilder().withRemoteServerHost(server).withRemoteServerPort(port).build());
   }
 
-  public RemoteHoodieTableFileSystemView(String server, int port, HoodieTableMetaClient metaClient, int timeoutSecs) {
+  public RemoteHoodieTableFileSystemView(HoodieTableMetaClient metaClient, FileSystemViewStorageConfig viewConf) {
     this.basePath = metaClient.getBasePath();
-    this.serverHost = server;
-    this.serverPort = port;
     this.mapper = new ObjectMapper();
     this.metaClient = metaClient;
     this.timeline = metaClient.getActiveTimeline().filterCompletedAndCompactionInstants();
-    this.timeoutSecs = timeoutSecs;
+    this.serverHost = viewConf.getRemoteViewServerHost();
+    this.serverPort = viewConf.getRemoteViewServerPort();
+    this.timeoutSecs = viewConf.getRemoteTimelineClientTimeoutSecs();
+    this.urlCheckedFunc = new HttpRequestCheckedFunction(this.timeoutSecs * 1000);
+    if (viewConf.isRemoteTimelineClientRetryEnabled()) {
+      retryHelper =  new RetryHelper(
+              viewConf.getRemoteTimelineClientMaxRetryIntervalMs(),
+              viewConf.getRemoteTimelineClientMaxRetryNumbers(),
+              viewConf.getRemoteTimelineInitialRetryIntervalMs(),
+              viewConf.getRemoteTimelineClientRetryExceptions(),
+              "Sending request");
+    }
   }
 
   private <T> T executeRequest(String requestPath, Map<String, String> queryParameters, TypeReference reference,
@@ -165,17 +179,9 @@ public class RemoteHoodieTableFileSystemView implements SyncableFileSystemView,
 
     String url = builder.toString();
     LOG.info("Sending request : (" + url + ")");
-    Response response;
-    int timeout = this.timeoutSecs * 1000; // msec
-    switch (method) {
-      case GET:
-        response = Request.Get(url).connectTimeout(timeout).socketTimeout(timeout).execute();
-        break;
-      case POST:
-      default:
-        response = Request.Post(url).connectTimeout(timeout).socketTimeout(timeout).execute();
-        break;
-    }
+    // Reset url and method, to avoid repeatedly instantiating objects.
+    urlCheckedFunc.setUrlAndMethod(url, method);
+    Response response =  retryHelper != null ? retryHelper.tryWith(urlCheckedFunc).start() : urlCheckedFunc.get();
     String content = response.returnContent().asString();
     return (T) mapper.readValue(content, reference);
   }
@@ -495,4 +501,33 @@ public class RemoteHoodieTableFileSystemView implements SyncableFileSystemView,
       throw new HoodieRemoteException(e);
     }
   }
+
+  /**
+   * For remote HTTP requests, to avoid repeatedly instantiating objects.
+   */
+  private class HttpRequestCheckedFunction implements RetryHelper.CheckedFunction<Response> {
+    private String url;
+    private RequestMethod method;
+    private final int timeoutMs;
+
+    public void setUrlAndMethod(String url, RequestMethod method) {
+      this.method = method;
+      this.url = url;
+    }
+    
+    public HttpRequestCheckedFunction(int timeoutMs) {
+      this.timeoutMs = timeoutMs;
+    }
+
+    @Override
+    public Response get() throws IOException {
+      switch (method) {
+        case GET:
+          return Request.Get(url).connectTimeout(timeoutMs).socketTimeout(timeoutMs).execute();
+        case POST:
+        default:
+          return Request.Post(url).connectTimeout(timeoutMs).socketTimeout(timeoutMs).execute();
+      }
+    }
+  }
 }
