@@ -66,6 +66,10 @@ public class HoodieCommonTestHarness {
     dataGen = new HoodieTestDataGenerator();
   }
 
+  protected void initTestDataGenerator(String[] partitionPaths) {
+    dataGen = new HoodieTestDataGenerator(partitionPaths);
+  }
+
   /**
    * Cleanups test data generator.
    *
