@@ -119,6 +119,20 @@ public class CompactionUtil {
     writeConfig.setSchema(tableAvroSchema.toString());
   }
 
+  /**
+   * Sets up the preCombine field into the given configuration {@code conf}
+   * through reading from the hoodie table metadata.
+   *
+   * This value is non-null as compaction can only be performed on MOR tables.
+   * Of which, MOR tables will have non-null precombine fields.
+   *
+   * @param conf The configuration
+   */
+  public static void setPreCombineField(Configuration conf, HoodieTableMetaClient metaClient) {
+    String preCombineField = metaClient.getTableConfig().getPreCombineField();
+    conf.setString(FlinkOptions.PRECOMBINE_FIELD, preCombineField);
+  }
+
   /**
    * Infers the changelog mode based on the data file schema(including metadata fields).
    *
