@@ -32,6 +32,7 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
 
+import static org.apache.hudi.common.util.PartitionPathEncodeUtils.DEFAULT_PARTITION_PATH;
 import static org.apache.hudi.utils.TestData.insertRow;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
@@ -54,19 +55,19 @@ public class TestRowDataKeyGen {
     final RowData rowData2 = insertRow(TestConfigurations.ROW_TYPE, null, StringData.fromString("Danny"), 23,
         TimestampData.fromEpochMillis(1), null);
     assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData2));
-    assertThat(keyGen1.getPartitionPath(rowData2), is("default"));
+    assertThat(keyGen1.getPartitionPath(rowData2), is(DEFAULT_PARTITION_PATH));
     // empty record key and partition path
     final RowData rowData3 = insertRow(StringData.fromString(""), StringData.fromString("Danny"), 23,
         TimestampData.fromEpochMillis(1), StringData.fromString(""));
     assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData3));
-    assertThat(keyGen1.getPartitionPath(rowData3), is("default"));
+    assertThat(keyGen1.getPartitionPath(rowData3), is(DEFAULT_PARTITION_PATH));
 
     // hive style partitioning
     conf.set(FlinkOptions.HIVE_STYLE_PARTITIONING, true);
     final RowDataKeyGen keyGen2 = RowDataKeyGen.instance(conf, TestConfigurations.ROW_TYPE);
-    assertThat(keyGen2.getPartitionPath(rowData1), is("partition=par1"));
-    assertThat(keyGen2.getPartitionPath(rowData2), is("partition=default"));
-    assertThat(keyGen2.getPartitionPath(rowData3), is("partition=default"));
+    assertThat(keyGen2.getPartitionPath(rowData1), is(String.format("partition=%s", "par1")));
+    assertThat(keyGen2.getPartitionPath(rowData2), is(String.format("partition=%s", DEFAULT_PARTITION_PATH)));
+    assertThat(keyGen2.getPartitionPath(rowData3), is(String.format("partition=%s", DEFAULT_PARTITION_PATH)));
   }
 
   @Test
@@ -83,19 +84,19 @@ public class TestRowDataKeyGen {
     // null record key and partition path
     final RowData rowData2 = insertRow(TestConfigurations.ROW_TYPE, null, null, 23, null, null);
     assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData2));
-    assertThat(keyGen1.getPartitionPath(rowData2), is("default/default"));
+    assertThat(keyGen1.getPartitionPath(rowData2), is(String.format("%s/%s", DEFAULT_PARTITION_PATH, DEFAULT_PARTITION_PATH)));
     // empty record key and partition path
     final RowData rowData3 = insertRow(StringData.fromString(""), StringData.fromString(""), 23,
         TimestampData.fromEpochMillis(1), StringData.fromString(""));
     assertThrows(HoodieKeyException.class, () -> keyGen1.getRecordKey(rowData3));
-    assertThat(keyGen1.getPartitionPath(rowData3), is("default/1970-01-01T00:00:00.001"));
+    assertThat(keyGen1.getPartitionPath(rowData3), is(String.format("%s/1970-01-01T00:00:00.001", DEFAULT_PARTITION_PATH)));
 
     // hive style partitioning
     conf.set(FlinkOptions.HIVE_STYLE_PARTITIONING, true);
     final RowDataKeyGen keyGen2 = RowDataKeyGen.instance(conf, TestConfigurations.ROW_TYPE);
-    assertThat(keyGen2.getPartitionPath(rowData1), is("partition=par1/ts=1970-01-01T00:00:00.001"));
-    assertThat(keyGen2.getPartitionPath(rowData2), is("partition=default/ts=default"));
-    assertThat(keyGen2.getPartitionPath(rowData3), is("partition=default/ts=1970-01-01T00:00:00.001"));
+    assertThat(keyGen2.getPartitionPath(rowData1), is(String.format("partition=%s/ts=%s", "par1", "1970-01-01T00:00:00.001")));
+    assertThat(keyGen2.getPartitionPath(rowData2), is(String.format("partition=%s/ts=%s", DEFAULT_PARTITION_PATH, DEFAULT_PARTITION_PATH)));
+    assertThat(keyGen2.getPartitionPath(rowData3), is(String.format("partition=%s/ts=%s", DEFAULT_PARTITION_PATH, "1970-01-01T00:00:00.001")));
   }
 
   @Test
