@@ -27,6 +27,7 @@ import org.apache.hudi.common.model.HoodieRecordPayload;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.exception.HoodieIndexException;
 import org.apache.hudi.metadata.MetadataPartitionType;
@@ -53,7 +54,6 @@ import static org.apache.hudi.common.util.StringUtils.isNullOrEmpty;
 import static org.apache.hudi.common.util.ValidationUtils.checkArgument;
 import static org.apache.hudi.metadata.HoodieTableMetadataUtil.PARTITION_NAME_BLOOM_FILTERS;
 import static org.apache.hudi.metadata.HoodieTableMetadataUtil.PARTITION_NAME_COLUMN_STATS;
-import static org.apache.hudi.metadata.HoodieTableMetadataUtil.getCompletedMetadataPartitions;
 import static org.apache.hudi.metadata.HoodieTableMetadataUtil.getInflightAndCompletedMetadataPartitions;
 import static org.apache.hudi.utilities.UtilHelpers.EXECUTE;
 import static org.apache.hudi.utilities.UtilHelpers.SCHEDULE;
@@ -227,7 +227,8 @@ public class HoodieIndexer {
   }
 
   private Option<String> doSchedule(SparkRDDWriteClient<HoodieRecordPayload> client) {
-    List<MetadataPartitionType> partitionTypes = getRequestedPartitionTypes(cfg.indexTypes);
+    HoodieMetadataConfig metadataConfig = getHoodieMetadataConfig();
+    List<MetadataPartitionType> partitionTypes = getRequestedPartitionTypes(cfg.indexTypes, Option.of(metadataConfig));
     checkArgument(partitionTypes.size() == 1, "Currently, only one index type can be scheduled at a time.");
     if (!isMetadataInitialized() && !partitionTypes.contains(MetadataPartitionType.FILES)) {
       throw new HoodieException("Metadata table is not yet initialized. Initialize FILES partition before any other partition " + Arrays.toString(partitionTypes.toArray()));
@@ -242,8 +243,14 @@ public class HoodieIndexer {
     return indexingInstant;
   }
 
+  private HoodieMetadataConfig getHoodieMetadataConfig() {
+    props.setProperty(HoodieWriteConfig.BASE_PATH.key(), cfg.basePath);
+    HoodieWriteConfig dataTableWriteConfig = HoodieWriteConfig.newBuilder().withProps(props).build();
+    return dataTableWriteConfig.getMetadataConfig();
+  }
+
   private boolean indexExists(List<MetadataPartitionType> partitionTypes) {
-    Set<String> indexedMetadataPartitions = getCompletedMetadataPartitions(metaClient.getTableConfig());
+    Set<String> indexedMetadataPartitions = metaClient.getTableConfig().getMetadataPartitions();
     Set<String> requestedIndexPartitionPaths = partitionTypes.stream().map(MetadataPartitionType::getPartitionPath).collect(Collectors.toSet());
     requestedIndexPartitionPaths.retainAll(indexedMetadataPartitions);
     if (!requestedIndexPartitionPaths.isEmpty()) {
@@ -254,7 +261,7 @@ public class HoodieIndexer {
   }
 
   private boolean isMetadataInitialized() {
-    Set<String> indexedMetadataPartitions = getCompletedMetadataPartitions(metaClient.getTableConfig());
+    Set<String> indexedMetadataPartitions = metaClient.getTableConfig().getMetadataPartitions();
     return !indexedMetadataPartitions.isEmpty();
   }
 
@@ -292,7 +299,7 @@ public class HoodieIndexer {
   }
 
   private int dropIndex(JavaSparkContext jsc) throws Exception {
-    List<MetadataPartitionType> partitionTypes = getRequestedPartitionTypes(cfg.indexTypes);
+    List<MetadataPartitionType> partitionTypes = getRequestedPartitionTypes(cfg.indexTypes, Option.empty());
     String schemaStr = UtilHelpers.getSchemaFromLatestInstant(metaClient);
     try (SparkRDDWriteClient<HoodieRecordPayload> client = UtilHelpers.createHoodieClient(jsc, cfg.basePath, schemaStr, cfg.parallelism, Option.empty(), props)) {
       client.dropIndex(partitionTypes);
@@ -317,16 +324,27 @@ public class HoodieIndexer {
   boolean isIndexBuiltForAllRequestedTypes(List<HoodieIndexPartitionInfo> indexPartitionInfos) {
     Set<String> indexedPartitions = indexPartitionInfos.stream()
         .map(HoodieIndexPartitionInfo::getMetadataPartitionPath).collect(Collectors.toSet());
-    Set<String> requestedPartitions = getRequestedPartitionTypes(cfg.indexTypes).stream()
+    Set<String> requestedPartitions = getRequestedPartitionTypes(cfg.indexTypes, Option.empty()).stream()
         .map(MetadataPartitionType::getPartitionPath).collect(Collectors.toSet());
     requestedPartitions.removeAll(indexedPartitions);
     return requestedPartitions.isEmpty();
   }
 
-  List<MetadataPartitionType> getRequestedPartitionTypes(String indexTypes) {
+  List<MetadataPartitionType> getRequestedPartitionTypes(String indexTypes, Option<HoodieMetadataConfig> metadataConfig) {
     List<String> requestedIndexTypes = Arrays.asList(indexTypes.split(","));
     return requestedIndexTypes.stream()
-        .map(p -> MetadataPartitionType.valueOf(p.toUpperCase(Locale.ROOT)))
-        .collect(Collectors.toList());
+        .map(p -> {
+          MetadataPartitionType metadataPartitionType = MetadataPartitionType.valueOf(p.toUpperCase(Locale.ROOT));
+          if (metadataConfig.isPresent()) { // this is expected to be non-null during scheduling where file groups for a given partition are instantiated for the first time.
+            if (!metadataPartitionType.getPartitionPath().equals(MetadataPartitionType.FILES.toString())) {
+              if (metadataPartitionType.getPartitionPath().equals(MetadataPartitionType.COLUMN_STATS.getPartitionPath())) {
+                metadataPartitionType.setFileGroupCount(metadataConfig.get().getColumnStatsIndexFileGroupCount());
+              } else if (metadataPartitionType.getPartitionPath().equals(MetadataPartitionType.BLOOM_FILTERS.getPartitionPath())) {
+                metadataPartitionType.setFileGroupCount(metadataConfig.get().getBloomFilterIndexFileGroupCount());
+              }
+            }
+          }
+          return metadataPartitionType;
+        }).collect(Collectors.toList());
   }
 }
